
package com.tmerclub.cloud.coupon.vo;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.tmerclub.cloud.common.product.vo.search.SpuSearchVO;
import com.tmerclub.cloud.common.vo.BaseVO;
import io.swagger.v3.oas.annotations.media.Schema;

import java.io.Serial;
import java.io.Serializable;
import java.util.Date;
import java.util.List;

/**
 * 优惠券VO
 *
 * @author YXF
 * @date 2020-12-08 17:22:56
 */
public class CouponVO extends BaseVO implements Serializable {
    @Serial
    private static final long serialVersionUID = 1L;
    @Schema(description = "优惠券ID")
    private Long couponId;

    @Schema(description = "店铺ID")
    private Long shopId;

    @Schema(description = "优惠券名称")
    private String couponName;

    @Schema(description = "副标题")
    private String subTitle;

    @Schema(description = "优惠类型 1:代金券 2:折扣券 3:兑换券")
    private Integer couponType;

    @Schema(description = "适用商品类型 0全部商品参与 1指定商品参与")
    private Integer suitableProdType;

    @Schema(description = "获取方式 0=用户领取 1=店铺发放")
    private Integer getWay;

    @Schema(description = "生效类型 1:固定时间 2:领取后生效")
    private Integer validTimeType;

    @Schema(description = "使用条件")
    private Long cashCondition;

    @Schema(description = "减免金额")
    private Long reduceAmount;

    @Schema(description = "折扣额度")
    private Double couponDiscount;

    @Schema(description = "投放时间")
    private Date launchTime;

    @Schema(description = "开始时间")
    private Date startTime;

    @Schema(description = "结束时间")
    private Date endTime;

    @Schema(description = "领券后X天起生效")
    private Integer afterReceiveDays;

    @Schema(description = "有效天数")
    private Integer validDays;

    @Schema(description = "总库存")
    private Integer totalStock;

    @Schema(description = "库存")
    private Integer stocks;

    @Schema(description = "每个用户领券上限，如不填则默认为1")
    private Integer limitNum;

    @Schema(description = "优惠券状态 0:过期 1:未过期")
    private Integer status;

    @Schema(description = "优惠券投放状态(-1:取消投放 0:自动投放 1:投放 2:违规下架 3:等待审核 4:暂不投放)")
    private Integer putonStatus;

    @Schema(description = "商品Id列表")
    private List<Long> spuIds;

    @Schema(description = "商品列表")
    private List<SpuSearchVO> spus;

    @Schema(description = "用户拥有的优惠券数量")
    private Integer couponCount;

    @Schema(description = "领券时间")
    @JsonFormat(pattern = "yyyy-MM-dd")
    private Date receiveTime;

    @Schema(description = "店铺名称")
    private String shopName;

    public Long getCouponId() {
        return couponId;
    }

    public void setCouponId(Long couponId) {
        this.couponId = couponId;
    }

    public Long getShopId() {
        return shopId;
    }

    public void setShopId(Long shopId) {
        this.shopId = shopId;
    }

    public String getCouponName() {
        return couponName;
    }

    public void setCouponName(String couponName) {
        this.couponName = couponName;
    }

    public String getSubTitle() {
        return subTitle;
    }

    public void setSubTitle(String subTitle) {
        this.subTitle = subTitle;
    }

    public Integer getCouponType() {
        return couponType;
    }

    public void setCouponType(Integer couponType) {
        this.couponType = couponType;
    }

    public Integer getSuitableProdType() {
        return suitableProdType;
    }

    public void setSuitableProdType(Integer suitableProdType) {
        this.suitableProdType = suitableProdType;
    }

    public Integer getGetWay() {
        return getWay;
    }

    public void setGetWay(Integer getWay) {
        this.getWay = getWay;
    }

    public Integer getValidTimeType() {
        return validTimeType;
    }

    public void setValidTimeType(Integer validTimeType) {
        this.validTimeType = validTimeType;
    }

    public Long getCashCondition() {
        return cashCondition;
    }

    public void setCashCondition(Long cashCondition) {
        this.cashCondition = cashCondition;
    }

    public Long getReduceAmount() {
        return reduceAmount;
    }

    public void setReduceAmount(Long reduceAmount) {
        this.reduceAmount = reduceAmount;
    }

    public Double getCouponDiscount() {
        return couponDiscount;
    }

    public void setCouponDiscount(Double couponDiscount) {
        this.couponDiscount = couponDiscount;
    }

    public Date getLaunchTime() {
        return launchTime;
    }

    public void setLaunchTime(Date launchTime) {
        this.launchTime = launchTime;
    }

    public Date getStartTime() {
        return startTime;
    }

    public void setStartTime(Date startTime) {
        this.startTime = startTime;
    }

    public Date getEndTime() {
        return endTime;
    }

    public void setEndTime(Date endTime) {
        this.endTime = endTime;
    }

    public Integer getAfterReceiveDays() {
        return afterReceiveDays;
    }

    public void setAfterReceiveDays(Integer afterReceiveDays) {
        this.afterReceiveDays = afterReceiveDays;
    }

    public Integer getValidDays() {
        return validDays;
    }

    public void setValidDays(Integer validDays) {
        this.validDays = validDays;
    }

    public Integer getTotalStock() {
        return totalStock;
    }

    public void setTotalStock(Integer totalStock) {
        this.totalStock = totalStock;
    }

    public Integer getStocks() {
        return stocks;
    }

    public void setStocks(Integer stocks) {
        this.stocks = stocks;
    }

    public Integer getLimitNum() {
        return limitNum;
    }

    public void setLimitNum(Integer limitNum) {
        this.limitNum = limitNum;
    }

    public Integer getStatus() {
        return status;
    }

    public void setStatus(Integer status) {
        this.status = status;
    }

    public Integer getPutonStatus() {
        return putonStatus;
    }

    public void setPutonStatus(Integer putonStatus) {
        this.putonStatus = putonStatus;
    }

    public List<Long> getSpuIds() {
        return spuIds;
    }

    public void setSpuIds(List<Long> spuIds) {
        this.spuIds = spuIds;
    }

    public List<SpuSearchVO> getSpus() {
        return spus;
    }

    public void setSpus(List<SpuSearchVO> spus) {
        this.spus = spus;
    }

    public Integer getCouponCount() {
        return couponCount;
    }

    public void setCouponCount(Integer couponCount) {
        this.couponCount = couponCount;
    }

    public Date getReceiveTime() {
        return receiveTime;
    }

    public void setReceiveTime(Date receiveTime) {
        this.receiveTime = receiveTime;
    }

    public String getShopName() {
        return shopName;
    }

    public void setShopName(String shopName) {
        this.shopName = shopName;
    }

    @Override
    public String toString() {
        return "CouponVO{" +
                "couponId=" + couponId +
                ", shopId=" + shopId +
                ", couponName='" + couponName + '\'' +
                ", subTitle='" + subTitle + '\'' +
                ", couponType=" + couponType +
                ", suitableProdType=" + suitableProdType +
                ", getWay=" + getWay +
                ", validTimeType=" + validTimeType +
                ", cashCondition=" + cashCondition +
                ", reduceAmount=" + reduceAmount +
                ", couponDiscount=" + couponDiscount +
                ", launchTime=" + launchTime +
                ", startTime=" + startTime +
                ", endTime=" + endTime +
                ", afterReceiveDays=" + afterReceiveDays +
                ", validDays=" + validDays +
                ", totalStock=" + totalStock +
                ", stocks=" + stocks +
                ", limitNum=" + limitNum +
                ", status=" + status +
                ", putonStatus=" + putonStatus +
                ", spuIds=" + spuIds +
                ", spus=" + spus +
                ", couponCount=" + couponCount +
                ", receiveTime=" + receiveTime +
                ", shopName='" + shopName + '\'' +
                '}';
    }
}
